@@ -34,7 +34,7 @@ partialrvdata::partialrvdata(vind nvariables)
 }
 
 rvdata::rvdata(vind lastvariab,vind nvtopiv,vind tnv,rvgdata *data,const deque<bool>& active,vind *origvarlist,real criterion)
-  :  lastv(lastvariab), p(tnv), k(nvtopiv), crt(criterion), varin(active), orgvar(origvarlist), e(0), gdt(data), rpl(0), unreliable(false)
+  :  lastv(lastvariab), p(tnv), k(nvtopiv), crt(criterion), varin(active), orgvar(origvarlist), e(0), unreliable(false), rpl(0), gdt(data)
 {
 	try {
 		if (k > 0)  {
@@ -443,3 +443,4 @@ real rvdata::frobenius(twodarray& m,bool *inlst) const
 }
 
 }
+
