@@ -3,7 +3,7 @@
        SUBROUTINE anneal(criterio,p,svector,kmin,kmax,valores,vars,
      + bestval,bestvar,nfora,fora,ndentro,dentro,nsol,niter,
      + improvement,beta,temp,coolfreq,nqsi,qsi,esp,silog,solinit,valp,
-     + vecvecp,hvector,rh)
+     + vecvecp,hvector,rh,checksg,numtol,numprb)
 *#######################################################################
 
 * Routine to perform a Simulated Annealing search for a k-variable subset
@@ -68,10 +68,15 @@
 * vecvecp  - double precision vector, giving the eigenvectors of the 
 *            covariance (or correlation) matrix of the p variables (given 
 *           as a vector for convenience in passing from R to Fortran). 
-* hvector  - double precision vector, giving the full effect descrption 
+* hvector  - double precision vector, giving the full effect description 
 *            matrix (H) of the p variables (given as a vector for
 *            convenience in passing from R to Fortran). 
 *   rh     - integer variable giving the expected rank of H matrix.
+* checksg - logical, flag indicating if a test for ill-conditioned 
+*	    (E being numerically singular) problems should be implemented
+*  numtol - double precision tolerance level for the ratio of the diagonal 
+*           elements of E Choleski decomposition
+
 * 
 *
 *
@@ -97,46 +102,51 @@
 *           default), then the subset given results from running the 
 *           restricted local search algorithm on the best solution from 
 *           Simulated Annealing (output for R/S).
+* numprb -  logical flag set to true if numerical problems preclude some
+*           (ill-conditioned) subsets to be evaluated, and to false otherwise.   
 *
 
 * general declarations
-       INTEGER i,j,k,m,p,criterio,niter,poriginal,rh
+       INTEGER i,j,k,m,p,criterio,kmin,kmax,nfora,ndentro,niter
+       INTEGER poriginal,coolfreq 
        INTEGER fora(0:nfora),fica(0:p),dentro(0:ndentro),auxw(kmax)
        INTEGER vars(nsol*(kmax-kmin+1)*kmax)
        INTEGER bestvar((kmax-kmin+1)*kmax)
-       LOGICAL setk(p),setkmax(p),improvement
-       DOUBLE PRECISION s(p,p),sq(p,p),svector(p*p),vecvecp(p*p),h(p,p)
-       DOUBLE PRECISION vmax,vactual,vcorrente
-       DOUBLE PRECISION valores((kmax-kmin+1)*nsol),hvector(p*p)
-       DOUBLE PRECISION beta,temp,critvalue
-       integer coolfreq 
-       DOUBLE PRECISION bestval(kmax-kmin+1)
-       LOGICAL silog
        INTEGER solinit(kmax*nsol*(kmax-kmin+1))
-* declarations only for the RM criterion
-       DOUBLE PRECISION tracos,dobjrm
-* declarations only for the RV criterion
-       DOUBLE PRECISION tracosq,dobjrv
+       DOUBLE PRECISION s(p,p),svector(p*p),vecvecp(p*p)
+       DOUBLE PRECISION vmax,vactual,vcorrente,numtol
+       DOUBLE PRECISION valores((kmax-kmin+1)*nsol)
+       DOUBLE PRECISION beta,temp,critvalue
+       DOUBLE PRECISION bestval(kmax-kmin+1)
+       LOGICAL silog,improvement,checksg,numprb
+       LOGICAL setk(p),setkmax(p)
+* declarations only for the RM and RV criteria
+       DOUBLE PRECISION tracos,tracosq,dobjrm,dobjrv
+       DOUBLE PRECISION sq(p,p)
 * declarations only for the GCD criterion
-       DOUBLE PRECISION valp(p), vecp(p,p),dobjgcd
+       DOUBLE PRECISION valp(p),vecp(p,p),dobjgcd
        INTEGER nqsi,qsi(p)
        LOGICAL esp
-* declarations for tau2,xi2,zeta2 e ccr12
-       Double precision dobjtau2,dobjxi2,dobjzeta2,dobjccr12
-
+* declarations only for the tau2,xi2,zeta2 and ccr12 criteria
+       INTEGER rh
+       DOUBLE PRECISION hvector(p*p),h(p,p)
+       DOUBLE PRECISION dobjtau2,dobjxi2,dobjzeta2,dobjccr12
+* declarations of local arrays and matrices
+       INTEGER randsk1pp(p),setint(p)
+       DOUBLE PRECISION skinput(kmax,kmax),hkinput(kmax,kmax)
+       DOUBLE PRECISION ekinput(kmax,kmax),workmat(kmax,kmax)
+       DOUBLE PRECISION work(6*kmax),egval(kmax)
 
        external randsk1,dobjrm,dobjrv,dobjgcd,dobjtau2,dobjxi2
        external dcorrigesk,dannealing,dobjzeta2,dobjccr12
        external dmelhoramentogen,newinicializar
 
-
-
 * initializations 
 
       call newinicializar(criterio,p,s,svector,sq,nfora,fora,ndentro,
      +  dentro,fica,tracos,tracosq,vecp,poriginal,vecvecp,
      +  h,hvector,rh)
-      critvalue = 0.0D0
+       critvalue = -1.0D0
 
 **********************************
 * The loop which is to be repeated for each cardinality of subsets, 
@@ -146,7 +156,7 @@
 
 * Generates nsol solutions via Simulated Annealing
 
-         if (criterio.eq.3) then
+	if (criterio.eq.3) then
              if (.not.esp) then
                nqsi=k
                do m=1,nqsi
@@ -154,13 +164,13 @@
                end do
              end if
          end if
-         vmax=0
+         vmax=-1
 
          do ksol=1,nsol
 
 * determining the initial solution.
 
-           if (silog) then
+          if (silog) then
              do m=1,poriginal
                setk(m)=.false.
              end do           
@@ -174,45 +184,47 @@
              end if  
            else
 
-               call randsk1(p-ndentro,k-ndentro,setk)  
+               call randsk1(p-ndentro,k-ndentro,setk,randsk1pp)  
                if(ndentro .GT. 0) then
                  call dcorrigesk(ndentro,dentro,p,setk,poriginal)
                endif
           endif
 
-
-
            if (criterio.eq.1) then
-               vactual=dobjrm(k,setk,p,s,sq,poriginal)
+		vactual=dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
+     +   setint,skinput)
            end if
            if (criterio.eq.2) then
-               vactual=dobjrv(k,setk,p,s,sq,poriginal)
+           	vactual=dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
+     +   setint,skinput)
            end if
            if (criterio.eq.3) then
-               vactual=dobjgcd(nqsi,qsi,valp,vecp,k,setk,p,s,
-     +           fica,poriginal)
+		vactual=dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
+     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
            end if
            if (criterio.eq.4) then
-               vactual=dobjtau2(k,setk,p,s,poriginal,h,rh)
+                vactual=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
+           end if		 
+	   if (criterio.eq.5) then
+                vactual=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,workmat)
+           end if		 
+	   if (criterio.eq.6) then
+             vactual=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
            end if
-		 
-		 if (criterio.eq.5) then
-               vactual=dobjxi2(k,setk,p,s,poriginal,h,rh)
-           end if
-		 
-		 if (criterio.eq.6) then
-               vactual=dobjzeta2(k,setk,p,s,poriginal,h,rh)
-           end if
-		 
-		 if (criterio.eq.7) then
+	   if (criterio.eq.7) then
 * tirei rh
-               vactual=dobjccr12(k,setk,p,s,poriginal,h)
+               vactual=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
+     +   numtol,setint,work,egval,skinput,hkinput,workmat)
            end if
-       
-           call dannealing(criterio,p,k,s,sq,setk,vactual,ndentro,
-     +       dentro,niter,beta,temp,coolfreq,tracos,tracosq,
-     +       nqsi,qsi,valp,vecp,fica,poriginal,h,rh)
-
+      
+           call dannealing(criterio,p,kmax,setk,vactual,ndentro,
+     +                dentro,niter,beta,temp,coolfreq,k,s,tracos,sq,
+     +                tracosq,nqsi,qsi,valp,vecp,fica,poriginal,
+     +                h,rh,skinput,hkinput,ekinput,egval,work,workmat,
+     +                randsk1pp,setint,checksg,numtol,numprb)
 
            if (criterio.eq.1) then
               critvalue = dsqrt(vactual/tracos)
@@ -230,7 +242,7 @@
 	     if (criterio.eq.5) then
                critvalue = vactual
            end if
-		  if (criterio.eq.6) then
+ 	      if (criterio.eq.6) then
               critvalue = vactual
            end if
 
@@ -239,12 +251,19 @@
            end if
 
 * preparing the output for R/S
+
             valores((k-kmin)*nsol+ksol)=critvalue
-            jjaux=0
+
+           jjaux=0
            do i=1,p
              if (setk(i)) then
                jjaux=jjaux+1
-               vars(nsol*kmax*(k-kmin)+nsol*(jjaux-1)+ksol) = fica(i)
+		if (critvalue.LT.0.0D0) then 
+                   vars(nsol*kmax*(k-kmin)+nsol*(jjaux-1)+ksol) = 0
+		else
+                   vars(nsol*kmax*(k-kmin)+nsol*(jjaux-1)+ksol) = 
+     +           fica(i)
+		end if
              end if
            end do  
 
@@ -256,12 +275,14 @@
            end if
          end do
          naux=0
+
          do j=1,p
            if(setkmax(j)) then
               naux=naux+1
               auxw(naux)= fica(j)
            end if
          end do
+
          if (criterio.eq.1) then
             critvalue=dsqrt(vmax/tracos)
          end if
@@ -287,14 +308,18 @@
              critvalue=vmax  
            end if
 
-* If improvement is "true" runs a restricted local improvement algorithm
-* on the best of the nsol solutions produced by Simulated Annealing.  
 
-         if (improvement) then
+* If improvement is "true" and a valid solution was found
+* runs a restricted local improvement algorithm on the 
+* best of the nsol solutions produced by Simulated Annealing.  
+
+         if (improvement.and.vmax.GE.0.D0) then
             vcorrente = vmax
 
-            call dmelhoramentogen(criterio,p,setkmax,vmax,ndentro,
-     +        dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,poriginal,h,rh)
+            call dmelhoramentogen(criterio,p,kmax,setk,vactual,ndentro,
+     +                dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,poriginal,
+     +                h,rh,skinput,hkinput,ekinput,egval,work,workmat,
+     +                randsk1pp,setint,checksg,numtol,numprb)
 
             if (vmax .GT. vcorrente) then 
                    naux=0
@@ -314,20 +339,17 @@
                        critvalue = vmax/dsqrt(DBLE(nqsi*k))
                    end if
                    if (criterio.eq.4) then
-					 critvalue=vmax
-				end if
-
-				if (criterio.eq.5) then
-				critvalue=vmax  
-				end if
-		 
-				if (criterio.eq.6) then
-				critvalue=vmax
-				end if
-
-				if (criterio.eq.7) then
-				critvalue=vmax  
-				end if
+			critvalue=vmax
+		   end if
+		   if (criterio.eq.5) then
+			critvalue=vmax  
+		   end if		 
+		   if (criterio.eq.6) then
+			critvalue=vmax
+		   end if
+		   if (criterio.eq.7) then
+			critvalue=vmax  
+		   end if
             end if      
          end if
 
